package com.example.myapplication;

import android.util.Log;

import okhttp3.*;
import java.io.IOException;
import java.util.Map;

public class Requests {
    private final OkHttpClient client;
    private final HttpUrl baseUrl;

    public Requests(String baseUrl) {
        this(baseUrl, new OkHttpClient());
    }

    public Requests(String baseUrl, OkHttpClient client) {
        this.client = client;
        this.baseUrl = HttpUrl.parse(baseUrl);
        if (this.baseUrl == null) {
            throw new IllegalArgumentException("Invalid base URL: " + baseUrl);
        }
    }

    public void get(String path, Map<String, String> headers,
                    Map<String, String> queryParams, Callback callback) {
        HttpUrl url = buildUrl(path, queryParams);
        Request request = buildRequest(url, headers).get().build();
        enqueue(request, callback);
    }

    public void post(String path, RequestBody body, Map<String, String> headers,
                     Map<String, String> queryParams, Callback callback) {
        HttpUrl url = buildUrl(path, queryParams);
        Request request = buildRequest(url, headers).post(body).build();
        enqueue(request, callback);
    }

    public void put(String path, RequestBody body, Map<String, String> headers,
                    Map<String, String> queryParams, Callback callback) {
        HttpUrl url = buildUrl(path, queryParams);
        Request request = buildRequest(url, headers).put(body).build();
        enqueue(request, callback);
    }

    public void delete(String path, Map<String, String> headers,
                       Map<String, String> queryParams, Callback callback) {
        HttpUrl url = buildUrl(path, queryParams);
        Request request = buildRequest(url, headers).delete().build();
        enqueue(request, callback);
    }

    private Request.Builder buildRequest(HttpUrl url, Map<String, String> headers) {
        Request.Builder builder = new Request.Builder().url(url);
        if (headers != null) {
            for (Map.Entry<String, String> entry : headers.entrySet()) {
                builder.addHeader(entry.getKey(), entry.getValue());
            }
        }
        return builder;
    }

    private HttpUrl buildUrl(String path, Map<String, String> queryParams) {
        // Remove leading slash to prevent double slashes
        String sanitizedPath = path.startsWith("/") ? path.substring(1) : path;

        HttpUrl.Builder urlBuilder = baseUrl.newBuilder()
                .addPathSegments(sanitizedPath);

        if (queryParams != null) {
            for (Map.Entry<String, String> entry : queryParams.entrySet()) {
                urlBuilder.addQueryParameter(entry.getKey(), entry.getValue());
            }
        }

        return urlBuilder.build();
    }

    private void enqueue(Request request, Callback callback) {
        Log.i("asd", String.valueOf(request));
        client.newCall(request).enqueue(callback);
    }

    // Optional simplified callback interface
    public interface SimpleCallback {
        void onSuccess(String responseBody);
        void onFailure(IOException e);
    }

    public void enqueueSimple(Request request, SimpleCallback callback) {
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                callback.onFailure(e);
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                try (ResponseBody responseBody = response.body()) {
                    if (response.isSuccessful() && responseBody != null) {
                        callback.onSuccess(responseBody.string());
                    } else {
                        callback.onFailure(new IOException("Unexpected code: " + response.code()));
                    }
                }
            }
        });
    }
}